#include <Wire.h>
#include <SeeedGrayOLED.h>
#include "AirQuality.h"
#include "Arduino.h"
#include "DHT.h"
#define DHTPIN 2     // A quelle broche se connecte DHT  

// Définition du type de configuration préférentielle
//#define DHTTYPE DHT11   // DHT 11
#define DHTTYPE DHT22   // DHT 22  (AM2302)
DHT dht(DHTPIN, DHTTYPE);

// Mise en service du capteur AIRQUALITYSENSOR
AirQuality airqualitysensor;
int current_quality = -1;
#define BUZZER 10

int dust_pin = 4;
unsigned long duration;
unsigned long starttime;
unsigned long sampletime_ms = 30000;//donc toutes les  30s ;
unsigned long lowpulseoccupancy = 0;
float ratio = 0;
float concentration = 0;

void setup()
{
  Wire.begin();    // Initialisation du cirtuit intéger
  SeeedGrayOled.init();  //initialisation ecran LCD
  SeeedGrayOled.clearDisplay();          // effacer l'écran et positionner le curseur en haut à gauche
  SeeedGrayOled.setNormalDisplay();      // choix de l'affichage de l'écran soit Cursseur Blanc sur ecran noir ou l'inverse.
  SeeedGrayOled.setVerticalMode();
  //SeeedGrayOled.setPageMode();           // initialisation des différentes pages d'affichage
  SeeedGrayOled.setTextXY(0, 0);         // initialise le curseur en position x=0 et y=0
  SeeedGrayOled.putString("Initializing"); //AFFICHE LA FIN DE LA L'INITIALISATION

  pinMode(BUZZER, OUTPUT);
  digitalWrite(BUZZER, LOW);
  airqualitysensor.init(14);
  dht.begin();

  pinMode(dust_pin, INPUT);
  starttime = millis();;
}

void loop()
{
  // lire la valeur de la température et de l'humidité toute les 250 ms
  // il est possible de lire le capteur jusqu'à toutes les 2 secondes
  float h = dht.readHumidity();
  float t = dht.readTemperature();
  SeeedGrayOled.setTextXY(8, 0);
  // vérifier que le retour des variables est valide. Si elles ne correspondent pas à des nombres alors il y un pb!
  if (isnan(t) || isnan(h))
  {
    SeeedGrayOled.putString(" Capteur t° & Humidité ERROR");  // afficher ERREUR
  }
  else
  {
    SeeedGrayOled.putString("Humid: ");         // Affichage de l'humidité
    SeeedGrayOled.setTextXY(9, 0);
    SeeedGrayOled.putString("   ");
    if (h >= 40 and h <= 50)                      // analyse la valeur de h (taux d'humidité en %)
      SeeedGrayOled.putString("le taux d'humidité est parfait");
    else if (h > 50)
      SeeedGrayOled.putString("l'air est trop sec");       // erreur  à corriger
    else if (h > 40)                                    // erreur  à corriger !
      SeeedGrayOled.putString("l'air est trop humide");    // erreur  à corriger !
    SeeedGrayOled.putFloat(h);
    SeeedGrayOled.putString("%");               // valeur en %
    SeeedGrayOled.setTextXY(10, 0);
    SeeedGrayOled.putString("Température dans le Hall.: ");         // affichage de la température intérieure
    SeeedGrayOled.setTextXY(11, 0);
    SeeedGrayOled.putString("   ");
    SeeedGrayOled.putFloat(t);
    SeeedGrayOled.putString("°F");              // Erreur
    if (t >= 19 and t <= 21)                      // il faut adapter les valeurs
      SeeedGrayOled.putString("la température dans le hall est parfaite");
    else if (t > 22)                                              // A modifier !
      SeeedGrayOled.putString("Il fait trop chaud dans le Hall");    // A modifier !
    else if (t < 19)                                              // A modifier !
      SeeedGrayOled.putString("Il fait trop froid dans le Hall");    // A modifier !
  }

  // Analyse de la qualité de l'Air
  current_quality = airqualitysensor.slope();
  if (current_quality >= 0) {
    if (current_quality <= 1)
      // Qualité de l'air est MAUVAISE si la valeur de la variable current_quality est.......
      SeeedGrayOled.setInverseDisplay();
    else
      SeeedGrayOled.setNormalDisplay();

    SeeedGrayOled.setTextXY(1, 0);
    SeeedGrayOled.putString("Qualité de l'AIR:"); // affiche sur l'écran
    SeeedGrayOled.setTextXY(2, 0);
    SeeedGrayOled.putString("   "); // vide la valeur numérique
    SeeedGrayOled.setTextXY(2, 0);
    SeeedGrayOled.putNumber(airqualitysensor.first_vol); // affiche la valeur de la variable airqualitysensor.first_vol

    SeeedGrayOled.setTextXY(0, 0); // Affiche la qualité de l'air
    if (current_quality == 0)
      SeeedGrayOled.putString("texte1");         //exemple de texte
    else if (current_quality == 1)
      SeeedGrayOled.putString("texte2");                            //exemple de texte
    else if (current_quality == 2)
      SeeedGrayOled.putString("texte3");                          // exemple de texte
    else if (current_quality == 3)
      SeeedGrayOled.putString("texte4");                         // exemple de texte

    if (current_quality == ?? ?? ?? ) {   // Mise en service du Buzer POUR indiquer les dangers
      digitalWrite(BUZZER, ?? ?? ?? ? );
      else if (current_quality == ?? ?? ? )
        digitalWrite(BUZZER, ?? ?? ?? );
      else
        digitalWrite(BUZZER, ?? ?? ?? ?? ?? ? );
    }
  }

  // Analyse des données du capteur de particules fines
  duration = pulseIn(dust_pin, LOW);
  lowpulseoccupancy = lowpulseoccupancy + duration;

  if ((millis() - starttime) > sampletime_ms) //si le temps d'échantillionnage est de 30s
  {
    ratio = lowpulseoccupancy / (sampletime_ms * 10.0); // pourcentage entier de 0=>100
    concentration = 1.1 * pow(ratio, 3) - 3.8 * pow(ratio, 2) + 520 * ratio + 0.62; // En utilisant la courbe de la fiche technique du capteur wiki
    /*Serial.print(lowpulseoccupancy);
      Serial.print(",");
      Serial.print(ratio);
      Serial.print(",");
      Serial.println(concentration);*/
    SeeedGrayOled.setTextXY(4, 0);
    SeeedGrayOled.putString("Particles:");
    SeeedGrayOled.setTextXY(5, 0);
    SeeedGrayOled.putString("    ");
    SeeedGrayOled.setTextXY(5, 0);
    SeeedGrayOled.putNumber(concentration);
    lowpulseoccupancy = 0;
    starttime = millis();
  }
}

ISR(TIMER2_OVF_vect)
{
  if (airqualitysensor.counter == 122) //set 2 seconds as a detected duty
  {

    airqualitysensor.last_vol = airqualitysensor.first_vol;
    airqualitysensor.first_vol = analogRead(A0);
    airqualitysensor.counter = 0;
    airqualitysensor.timer_index = 1;
    //PORTB=PORTB^0x20;
  }
  else
  {
    airqualitysensor.counter++;
  }
}